/* Modèle d'Ising 2D
   V. MAGNIN, 28 décembre 2003, Polytech Lille
   Dernière modification : 8 avril 2013 (java 7)

   La procédure de modélisation est tout à la fin  */

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
//import com.sun.image.codec.jpeg.*;

public class Ising2D {
 static FenetrePrincipale miaFenestro;
 static Panneau pano; 
 static JProgressBar progression;
 static BufferedImage monImage;
 static WritableRaster tamponImage;
 static int LARGEUR=400;   //Taille de l'image en pixels
 static int HAUTEUR=400;
 
 //Variables globales du modèle :
 static int l, h ; //Largeur et hauteur de l'image
    static int n;  // nb de particules
    static double energie, aimantation;
    // Couleurs de base :
    static int [] blanc = new int[]{255, 255, 255};
    static int [] noir = new int[]{0, 0, 0};
    static int [] couleur0= new int [3];
    static int [] couleur1= new int [3];
            
 public static void main (String args[]) throws IOException {
  miaFenestro=new FenetrePrincipale ("Modèle d'Ising 2D");
 }

 //Classe interne du panneau d'affichage :
  static class Panneau extends JPanel
 {
        //méthode appelée pour dessiner le contenu du panneau
        public void paintComponent (Graphics g)
     {
            //     super.paintComponent(g);
            g.drawImage(monImage, 0, 0, this);
        }
    }
  
    //Classe interne de la fenêtre :
 static class FenetrePrincipale extends JFrame implements KeyListener {
  static JTextField champTexte1;
        static JTextField champTexte2;
        static JTextField champTexte3;
    
  //Constructeur :
  FenetrePrincipale(String s) throws IOException {
            super(s); //Titre de la fenêtre

   //Si on veut fermer la fenêtre :
   addWindowListener(new WindowAdapter() {
    public void windowClosing(WindowEvent e) {
     System.exit(0);
    }
   });

   setBounds(0,0,800,550); // Position et taille de la fenêtre

   GridBagLayout grilleConteneur= new GridBagLayout();
   GridBagConstraints contraintes= new GridBagConstraints();
   JPanel panneau = new JPanel();
   panneau.setLayout(grilleConteneur);

            //Champs pour entrer la température :
   construireContraintes(contraintes, 0, 0, 1, 1, 10, 10);
   contraintes.fill=GridBagConstraints.NONE;
   JLabel label1=new JLabel("Température en K :", JLabel.LEFT);
   grilleConteneur.setConstraints(label1, contraintes);
   panneau.add(label1);

   construireContraintes(contraintes, 1, 0, 1, 1, 90, 10);
   contraintes.fill=GridBagConstraints.NONE;
   champTexte1 = new JTextField("273",20);
   grilleConteneur.setConstraints(champTexte1, contraintes);
   panneau.add(champTexte1);

            //Champs pour entrer le nombre d'itérations :
   construireContraintes(contraintes, 0, 1, 1, 1, 10, 10);
   contraintes.fill=GridBagConstraints.NONE;
   JLabel label2=new JLabel("Nombre d'itérations :", JLabel.LEFT);
   grilleConteneur.setConstraints(label2, contraintes);
   panneau.add(label2);

   construireContraintes(contraintes, 1, 1, 1, 1, 90, 10);
   contraintes.fill=GridBagConstraints.NONE;
   champTexte2 = new JTextField("1000000",20);
   grilleConteneur.setConstraints(champTexte2, contraintes);
   panneau.add(champTexte2);
   
   //Champs pour entrer le nombre de particules :
   construireContraintes(contraintes, 0, 2, 1, 1, 10, 10);
   contraintes.fill=GridBagConstraints.NONE;
   JLabel label3=new JLabel(" ", JLabel.LEFT);
   grilleConteneur.setConstraints(label3, contraintes);
   panneau.add(label3);

   construireContraintes(contraintes, 1, 2, 1, 1, 90, 10);
   contraintes.fill=GridBagConstraints.NONE;
   champTexte3 = new JTextField(" ",20);
   grilleConteneur.setConstraints(champTexte3, contraintes);
   panneau.add(champTexte3);
 
            //Bouton
   construireContraintes(contraintes, 0, 3, 1, 1, 0, 80);
   contraintes.fill=GridBagConstraints.NONE;
   JButton boutton1 = new JButton("Simuler !");
   grilleConteneur.setConstraints(boutton1, contraintes);
   panneau.add(boutton1);

            //Panneau d'affichage graphique :
   construireContraintes(contraintes, 1, 3, 1, 1, 90, 10);
   contraintes.fill=GridBagConstraints.NONE;
         pano= new Panneau(); //on déclare un panneau pour dessiner
         Dimension dim = new Dimension(LARGEUR,HAUTEUR);
         pano.setPreferredSize(dim);
   grilleConteneur.setConstraints(pano, contraintes);
   panneau.add(pano);   

            monImage=new BufferedImage(LARGEUR,HAUTEUR,BufferedImage.TYPE_INT_RGB);
            tamponImage=monImage.getRaster();
      
   //Si on clique sur le bouton :
   boutton1.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
     try {
      modeliser();
     }
     catch (IOException ee) {
      System.out.println("Erreur d'entrée/sortie !");
     }
    }
   });

   //Si on appuie sur Entrée :
   champTexte1.addKeyListener(this);
   champTexte2.addKeyListener(this);
   champTexte3.addKeyListener(this);

            //Barre de progression :
   construireContraintes(contraintes, 0, 4, 1, 1, 0, 10);
   contraintes.fill=GridBagConstraints.NONE;
   progression= new JProgressBar(0, 100);
   progression.setValue(0);
   progression.setStringPainted(true);
   grilleConteneur.setConstraints(progression, contraintes);
   panneau.add(progression);
   
   setContentPane(panneau);
   
   //Création du menu :
   JMenuBar barreDeMenu = new JMenuBar();

   JMenu menu1 = new JMenu("Commandes", true);
   JMenuItem menu1_Item1 = new JMenuItem("Quitter");
   menu1_Item1.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
     System.exit(0);
    }
   });
   menu1.add(menu1_Item1);

            JMenu menu2 = new JMenu("Initialisation", true);
   JMenuItem menu2_Item1 = new JMenuItem("Aléatoire");
   menu2_Item1.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
                    initialisation_generale();
                    initialisation_aleatoire();
    }
   });
   menu2.add(menu2_Item1);
   
   JMenuItem menu2_Item2 = new JMenuItem("Mi-noir mi-blanc");
   menu2_Item2.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
                    initialisation_generale();
                    initialisation_minoir();
    }
   });
   menu2.add(menu2_Item2);

   JMenuItem menu2_Item3 = new JMenuItem("Noir");
   menu2_Item3.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
                    initialisation_generale();
                    initialisation_noir();
    }
   });
   menu2.add(menu2_Item3);

            JMenuItem menu2_Item4 = new JMenuItem("Blanc");
   menu2_Item4.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
                    initialisation_generale();
                    initialisation_blanc();
    }
   });
   menu2.add(menu2_Item4);
   
   JMenu menu3 = new JMenu("Aide", true);
   JMenuItem menu3_Item1 = new JMenuItem("A propos...");
   menu3_Item1.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
     JOptionPane.showMessageDialog(null, "Modèle d'Ising 2D (28-12-2003)\n");
    }
   });
   menu3.add(menu3_Item1);

   barreDeMenu.add(menu1);
   barreDeMenu.add(menu2);
   barreDeMenu.add(menu3);
   setJMenuBar(barreDeMenu);

   setVisible(true);
  }

  //**********************
  // Gestion du clavier :
  //**********************
  void construireContraintes(GridBagConstraints gbc, int gx, int gy, int gw, int gh, int wx, int wy) {
   gbc.gridx=gx;
   gbc.gridy=gy;
   gbc.gridwidth=gw;
   gbc.gridwidth=gh;
   gbc.weightx=wx;
   gbc.weighty=wy;
  }

  //**********************
  // Gestion du clavier :
  //**********************
  public void keyReleased(KeyEvent entree) {
   int touche = entree.getKeyCode();

   //Si on relache la touche Entrée :
   if (touche==10) {
    try {
     modeliser();
    }
    catch (IOException ee) {
     System.out.println("Erreur d'entrée/sortie !");
    }
   }

  }

  public void keyTyped(KeyEvent entree) {
   //Rien
  }

  public void keyPressed(KeyEvent entree) {
   //Rien
  }

     //******************************************************
    // Modèle d'Ising 2D
    //******************************************************
        static void initialisation_generale () {
            //Largeur et hauteur de l'image:
            l = LARGEUR-1;
            h = HAUTEUR-1;
            n = LARGEUR*HAUTEUR;
        }

     static void initialisation_noir () {
           int i,j;

           for (i=0;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
                    tamponImage.setPixel(i,j, noir);
            }
           }
            pano.paintImmediately(0,0,LARGEUR,HAUTEUR);
        }

     static void initialisation_blanc () {
           int i,j;

           for (i=0;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
                    tamponImage.setPixel(i,j, blanc);
            }
           }
            pano.paintImmediately(0,0,LARGEUR,HAUTEUR);
        }

        static void initialisation_aleatoire () {
           int i,j;

           for (i=0;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
                    if (Math.random()< 0.5) {
                        tamponImage.setPixel(i,j, blanc);
                    }
                    else
                    {
                        tamponImage.setPixel(i,j, noir);
                    }
            }
           }
            pano.paintImmediately(0,0,LARGEUR,HAUTEUR);
        }

     static void initialisation_minoir () {
           int i,j;

           // On met la moitié des pixels en blanc :
           for (i=0;i<=l/2;i=i+1) {
            for (j=0;j<=h;j=j+1) {
              tamponImage.setPixel(i,j, blanc);
            }
           }
           // On met la moitié des pixels en noir :
           for (i=l/2+1;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
              tamponImage.setPixel(i,j, noir);
            }
           }
            pano.paintImmediately(0,0,LARGEUR,HAUTEUR);
        }

        static long calcul_aimantation () {
            long M=0;
            int i,j;

           for (i=0;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
                    tamponImage.getPixel(i,j, couleur0);
                    if (couleur0[0]==0) {
                        M=M-1;
                    }
                    else
                    {
                        M=M+1;
                    }
                }
            }

           //System.out.println("M="+M);
           return M;
        }

        static long calcul_energie () {
            long E=0;
            int i,j;

           for (i=0;i<=l;i=i+1) {
            for (j=0;j<=h;j=j+1) {
                    tamponImage.getPixel(i,       j, couleur0);
                    tamponImage.getPixel((i+1)%LARGEUR, j, couleur1);
                    if (couleur0[0]!=couleur1[0]) {
                        E=E+1;
                    }
                    else
                    {
                        E=E-1;
                    }
                    
                    tamponImage.getPixel(i, (j+1)%HAUTEUR, couleur1);
                    if (couleur0[0]!=couleur1[0]) {
                        E=E+1;
                    }
                    else
                    {
                        E=E-1;
                    }
             }
           }
           
           //System.out.println("E="+E);
           return E;
        }
        
        //Calcule la variation d'énergie du système quand on inverse le spin
        //de coordonnées (x,y)
        static int calcul_deltaE (int x, int y) {
            int deltaE=0;
            int xprec,xsuiv,yprec,ysuiv;
            
            xprec=x-1;
            xsuiv=x+1;
            yprec=y-1;
            ysuiv=y+1;
            
            //Conditions aux limites :
            if (xprec<0) {
                xprec=l;
            }
            else if (xsuiv>l) {
                xsuiv=0;
            }
            
            if (yprec<0) {
                yprec=h;
            }
            else if (ysuiv>h) {
                ysuiv=0;
            }
            
            tamponImage.getPixel(x, y, couleur0);
            tamponImage.getPixel(xprec, y, couleur1);
            if (couleur0[0]==couleur1[0]) {
                deltaE=deltaE+2;
            }
            else
            {
                deltaE=deltaE-2;
            }
            tamponImage.getPixel(xsuiv, y, couleur1);
            if (couleur0[0]==couleur1[0]) {
                deltaE=deltaE+2;
            }
            else
            {
                deltaE=deltaE-2;
            }
            tamponImage.getPixel(x, yprec, couleur1);
            if (couleur0[0]==couleur1[0]) {
                deltaE=deltaE+2;
            }
            else
            {
                deltaE=deltaE-2;
            }
            tamponImage.getPixel(x, ysuiv, couleur1);
            if (couleur0[0]==couleur1[0]) {
                deltaE=deltaE+2;
            }
            else
            {
                deltaE=deltaE-2;
            }
            
            return deltaE;
        }

        static void modeliser () throws IOException {
            int x,y; //coordonnées d'une particule
            double temperature;  //Température du matériau en Kelvin
            long t;       //Compteur de boucle
            long tmax; //Nombre d'itérations
           int i,j;
            int deltaE;
            
            // Couleurs de base :
            int [] blanc = new int[]{255, 255, 255};
            int [] noir = new int[]{0, 0, 0};

            energie=calcul_energie();
            aimantation=calcul_aimantation();
            
            long tempsDernierAffichage=System.currentTimeMillis();
            //Barre de progression à 0 :
            progression.setValue(0);
            progression.paintImmediately(0,0,progression.getWidth(),progression.getHeight());

            //On lit la température, le nombre d'itérations :
            temperature = Double.parseDouble(champTexte1.getText())/100.0;
            tmax = Integer.parseInt(champTexte2.getText());

            long dt_fichier=tmax/400;   //Période d'enregistrement du fichier de sortie
            if (dt_fichier==0) {
              dt_fichier=1;
            }
            long dt_affichage=tmax/100; //Période pour l'affichage graphique
            if (dt_affichage==0) {
              dt_affichage=1;
            }

             //Ouverture du fichier de sortie :
           FileWriter fichierRmax= new FileWriter("Ising2D.txt");


            for (t=0; t<=tmax; t++) {
                //On tire au hasard un spin :
                x=(int)(Math.random()*LARGEUR);
                y=(int)(Math.random()*HAUTEUR);
                deltaE=calcul_deltaE(x,y);
                
                if (Math.random()<(Math.exp(-deltaE/temperature))) {
                    energie=energie+deltaE;

                    tamponImage.getPixel(x,y, couleur0);
                    if (couleur0[0]==0) {
                        tamponImage.setPixel(x,y, blanc);
                        aimantation=aimantation+2;
                    }
                    else
                    {
                        tamponImage.setPixel(x,y, noir);
                        aimantation=aimantation-2;
                    }
                    //System.out.println("verif:"+x+" "+y+" "+energie+" "+calcul_energie()+" "+calcul_aimantation());
                }

                // On affiche l'image toutes les 40 ms :
           if ((System.currentTimeMillis()-tempsDernierAffichage)>=40) {
                    pano.paintImmediately(0,0,LARGEUR,HAUTEUR);
                    tempsDernierAffichage=System.currentTimeMillis();
                }
                //On met à jour la barre de progression régulièrement :
                if (t%dt_affichage==0) {
                    progression.setValue((int)((t*100.0)/tmax));
                   progression.paintImmediately(0,0,progression.getWidth(),progression.getHeight());
                   System.out.println(t+" "+energie+" "+aimantation);
                }

                // Régulièrement on enregistre les données calculées:
                if (t%dt_fichier == 0) {
                    fichierRmax.write(t+" "+energie+" "+aimantation+"\n");
                }
            } // Fin de la boucle temporelle

            // On ferme le fichier :
            fichierRmax.close();
            System.out.println(t+" "+energie+" "+aimantation);
            System.out.println("vérification :"+calcul_energie()+" "+calcul_aimantation());

            // On enregistre l'image au format JPEG :
/*            FileOutputStream sortie=new FileOutputStream(new File("Ising2D.jpg"));
            JPEGImageEncoder monEncodeur=JPEGCodec.createJPEGEncoder(sortie);
            JPEGEncodeParam param=monEncodeur.getDefaultJPEGEncodeParam(monImage);
            param.setQuality(1f, false);    //Sans compression
            monEncodeur.setJPEGEncodeParam(param);
            monEncodeur.encode(monImage);
            sortie.close(); */
    }  // Fin de la procédure de modélisation
    
  } // Fin de la classe interne "FenetrePrincipale"

} // Fin de la classe Ising2D

